// SPDX-FileCopyrightText: 2017 Friedrich W. H. Kossebau <kossebau@kde.org>
// SPDX-FileCopyrightText: 2025 Joshua Goins <josh@redstrate.com>
// SPDX-License-Identifier: LGPL-2.0-or-later

#include "kpartview.h"
#include "sitepreviewplugin.h"
#include "sitepreviewplugin_debug.h"

#include <KTextEditor/Document>
#include <KTextEditor/MainWindow>
#include <KActionCollection>
#include <KPluginFactory>
#include <KParts/NavigationExtension>

#include <QEvent>
#include <QFileInfo>
#include <QKeyEvent>
#include <QKeySequence>
#include <QLabel>
#include <QProcess>
#include <QWebEngineView>
#include <QWebEngineSettings>
#include <QMenu>

class CustomWebEngineView : public QWebEngineView {
public:
Q_SIGNALS:
    void contextMenuEvent(QContextMenuEvent *event) override {
        auto menu = createStandardContextMenu();
        menu->removeAction(pageAction(QWebEnginePage::SavePage));
        menu->removeAction(pageAction(QWebEnginePage::ViewSource));

        menu->popup(event->globalPos());
    }
};

KPartView::KPartView(SitePreviewPlugin *plugin, KTextEditor::MainWindow *mainWindow, QObject *parent)
    : QObject(parent)
    , m_mainWindow(mainWindow)
    , m_plugin(plugin)
{
    m_part = new CustomWebEngineView();
    m_part->settings()->setAttribute(QWebEngineSettings::ErrorPageEnabled, false);
    m_part->settings()->setAttribute(QWebEngineSettings::FullScreenSupportEnabled, false);

    connect(m_part, &QWebEngineView::loadFinished, this, [this](bool ok) {
        if (!ok) {
            m_part->setHtml(QStringLiteral("<p>Failed to connect to local server. Is Hugo running?</p>"));
        }
    });
}

KPartView::~KPartView()
{
    delete m_errorLabel;
}

QWidget *KPartView::widget() const
{
    return m_part;
}

KTextEditor::Document *KPartView::document() const
{
    return m_document;
}

bool KPartView::isAutoUpdating() const
{
    return m_autoUpdating;
}

void KPartView::setDocument(KTextEditor::Document *document)
{
    if (m_document == document) {
        return;
    }
    if (!m_part) {
        return;
    }

    m_document = document;

    if (m_document) {
        m_previewDirty = true;
        updatePreview();
    } else {
        m_part->setUrl({});
    }
}

void KPartView::setAutoUpdating(bool autoUpdating)
{
    if (m_autoUpdating == autoUpdating) {
        return;
    }

    m_autoUpdating = autoUpdating;

    if (m_autoUpdating) {
        if (m_document && m_part && m_previewDirty) {
            updatePreview();
        }
    }
}

std::optional<QUrl> KPartView::getURLFromFile(const QString &filePath)
{
    auto view = m_mainWindow->pluginView(QStringLiteral("kateprojectplugin"));
    QString baseDir = view->property("projectBaseDir").toString();

    return getURLFromFileHugo(filePath, baseDir);
}

void KPartView::updatePreview()
{
    if (!m_part->isVisible()) {
        return;
    }

    if (auto url = getURLFromFile(m_document->url().path())) {
        m_part->setUrl(*url);
    } else {
        m_part->setHtml(QStringLiteral("<p>The associated page could not be found!</p>"));
    }
}

bool KPartView::eventFilter(QObject *object, QEvent *event)
{
    if (object == m_part && event->type() == QEvent::Show) {
        if (m_document && m_autoUpdating && m_previewDirty) {
            updatePreview();
        }
        return true;
    }

    return QObject::eventFilter(object, event);
}

std::optional<QUrl> KPartView::getURLFromFileHugo(const QString &filePath, const QString &baseDir)
{
    const auto hugoProcess = new QProcess();
    hugoProcess->setProgram(QStringLiteral("hugo"));
    hugoProcess->setWorkingDirectory(baseDir);
    hugoProcess->setArguments({QStringLiteral("list"), QStringLiteral("all")});
    hugoProcess->start();
    hugoProcess->waitForFinished();

    QString output = QString::fromUtf8(hugoProcess->readAllStandardOutput());
    QTextStream s1(&output);
    s1.readLine(); // header

    auto documentPath = filePath;
    documentPath = documentPath.remove(baseDir + QLatin1Char('/'));

    while (!s1.atEnd()){
        QString line = s1.readLine(); // reads line from file
        QStringList columns = line.split(QLatin1Char(','));

        if (columns.constFirst().contains(QStringLiteral("%1").arg(documentPath))) {
            auto actualUrl = QUrl::fromUserInput(columns[7]);
            actualUrl.setScheme(QStringLiteral("http"));
            actualUrl.setHost(QStringLiteral("localhost"));
            actualUrl.setPort(m_plugin->port());

            return actualUrl;
        }
    }

    return std::nullopt;
}

QUrl KPartView::url() const
{
    return m_part->url();
}
