// SPDX-FileCopyrightText: 2025 Joshua Goins <josh@redstrate.com>
// SPDX-License-Identifier: LGPL-2.0-or-later

#include "previewwidget.h"

#include "kpartview.h"
#include "sitepreviewplugin.h"
#include <sitepreviewplugin_debug.h>

#include <KAboutPluginDialog>
#include <KConfigGroup>
#include <KGuiItem>
#include <KLocalizedString>
#include <KParts/PartLoader>
#include <KPluginMetaData>
#include <KSharedConfig>
#include <KTextEditor/Document>
#include <KTextEditor/MainWindow>
#include <KTextEditor/View>
#include <KToggleAction>
#include <KXMLGUIFactory>

#include <QAction>
#include <QDesktopServices>
#include <QDomElement>
#include <QIcon>
#include <QLabel>
#include <QMenu>
#include <QToolButton>

PreviewWidget::PreviewWidget(SitePreviewPlugin *plugin, KTextEditor::MainWindow *mainWindow, QWidget *parent)
    : QStackedWidget(parent)
    , m_mainWindow(mainWindow)
    , m_plugin(plugin)
{
    m_lockAction = new KToggleAction(QIcon::fromTheme(QStringLiteral("object-unlocked-symbolic")), i18n("Lock Current Document"), this);
    m_lockAction->setToolTip(i18n("Lock preview to current document"));
    m_lockAction->setCheckedState(KGuiItem(i18n("Unlock Current View"), QIcon::fromTheme(QStringLiteral("object-locked")), i18n("Unlock current view")));
    m_lockAction->setChecked(false);
    connect(m_lockAction, &QAction::triggered, this, &PreviewWidget::toggleDocumentLocking);
    addAction(m_lockAction);

    m_openExternallyAction = new QAction(QIcon::fromTheme(QStringLiteral("open-link-symbolic")), i18n("Open Externally"), this);
    m_openExternallyAction->setToolTip(i18n("Open the current view in the default web browser"));
    connect(m_openExternallyAction, &QAction::triggered, this, [this] {
        QDesktopServices::openUrl(m_partView->url());
    });
    addAction(m_openExternallyAction);

    auto label = new QLabel(i18n("No preview available."));
    label->setAlignment(Qt::AlignHCenter);
    addWidget(label);

    connect(m_mainWindow, &KTextEditor::MainWindow::viewChanged, this, &PreviewWidget::setTextEditorView);

    setTextEditorView(m_mainWindow->activeView());
}

void PreviewWidget::readSessionConfig(const KConfigGroup &configGroup)
{
    m_lockAction->setChecked(configGroup.readEntry("documentLocked", false));
}

void PreviewWidget::writeSessionConfig(KConfigGroup &configGroup) const
{
    configGroup.writeEntry("documentLocked", m_lockAction->isChecked());
}

void PreviewWidget::setTextEditorView(KTextEditor::View *view)
{
    if ((view && view == m_previewedTextEditorView && view->document() == m_previewedTextEditorDocument
         && (!m_previewedTextEditorDocument))
        || !view || !isVisible() || m_lockAction->isChecked()) {
        return;
    }

    m_previewedTextEditorView = view;
    m_previewedTextEditorDocument = view->document();

    resetTextEditorView(m_previewedTextEditorDocument);
}

void PreviewWidget::resetTextEditorView(KTextEditor::Document *document)
{
    if (!isVisible() || m_previewedTextEditorDocument != document) {
        return;
    }

    m_partView = new KPartView(m_plugin, m_mainWindow, this);
    int index = addWidget(m_partView->widget());
    setCurrentIndex(index);

    if (m_partView) {
        m_partView->setDocument(m_previewedTextEditorDocument);
    }
}

void PreviewWidget::unsetDocument(KTextEditor::Document *document)
{
    if (!m_partView || m_previewedTextEditorDocument != document) {
        return;
    }

    m_partView->setDocument(nullptr);
    m_previewedTextEditorDocument = nullptr;

    // remove any current partview
    m_partView = nullptr;
}

void PreviewWidget::showEvent(QShowEvent *event)
{
    Q_UNUSED(event);

    if (m_lockAction->isChecked()) {
        resetTextEditorView(m_previewedTextEditorDocument);
    } else {
        setTextEditorView(m_mainWindow->activeView());
    }
}

void PreviewWidget::hideEvent(QHideEvent *event)
{
    Q_UNUSED(event);

    unsetDocument(m_previewedTextEditorDocument);
}

void PreviewWidget::toggleDocumentLocking(bool locked)
{
    if (!locked) {
        setTextEditorView(m_mainWindow->activeView());
    }
}

void PreviewWidget::toggleAutoUpdating(bool autoRefreshing)
{
    if (!m_partView) {
        // nothing to do
        return;
    }

    m_partView->setAutoUpdating(autoRefreshing);
}

void PreviewWidget::updatePreview()
{
    if (m_partView && m_partView->document()) {
        m_partView->updatePreview();
    }
}
