// SPDX-FileCopyrightText: 2025 Joshua Goins <josh@redstrate.com>
// SPDX-License-Identifier: LGPL-2.0-or-later

#include "sitepreviewview.h"

#include "previewwidget.h"
#include "sitepreviewplugin.h"

#include <KLocalizedString>
#include <KTextEditor/MainWindow>
#include <KActionCollection>
#include <KXMLGUIFactory>

#include <QIcon>
#include <QLayout>

KTextEditorPreviewView::KTextEditorPreviewView(SitePreviewPlugin *plugin, KTextEditor::MainWindow *mainWindow)
    : QObject(mainWindow)
    , m_mainWindow(mainWindow)
    , m_plugin(plugin)
{
    KXMLGUIClient::setComponentName(QStringLiteral("sitepreviewplugin"), i18n("Site Preview"));
    setXMLFile(QStringLiteral("ui.rc"));

    m_toolView = mainWindow->createToolView(plugin,
                                            QStringLiteral("sitepreviewplugin"),
                                            KTextEditor::MainWindow::Right,
                                            QIcon::fromTheme(QStringLiteral("globe-symbolic")),
                                            i18n("Site Preview"));

    // add preview widget
    m_previewView = new PreviewWidget(plugin, mainWindow, m_toolView.data());
    m_toolView->layout()->setContentsMargins(0, 0, 0, 0);
    m_toolView->layout()->addWidget(m_previewView);
    m_toolView->addActions(m_previewView->actions());

    m_startHugoAction = actionCollection()->addAction(QStringLiteral("start_site_preview"));
    m_startHugoAction->setText(i18nc("@action", "Start Website"));
    connect(m_startHugoAction, &QAction::triggered, this, &KTextEditorPreviewView::startHugo);

    m_stopHugoAction = actionCollection()->addAction(QStringLiteral("stop_site_preview"));
    m_stopHugoAction->setText(i18nc("@action", "Stop Website"));
    m_stopHugoAction->setEnabled(false);
    connect(m_stopHugoAction, &QAction::triggered, this, &KTextEditorPreviewView::stopHugo);

    mainWindow->guiFactory()->addClient(this);

    m_hugoProcess.setProgram(QStringLiteral("hugo"));
}

KTextEditorPreviewView::~KTextEditorPreviewView()
{
    delete m_toolView;
}

void KTextEditorPreviewView::readSessionConfig(const KConfigGroup &config)
{
    m_previewView->readSessionConfig(config);
}

void KTextEditorPreviewView::writeSessionConfig(KConfigGroup &config)
{
    m_previewView->writeSessionConfig(config);
}

void KTextEditorPreviewView::startHugo()
{
    auto view = m_mainWindow->pluginView(QStringLiteral("kateprojectplugin"));
    QString baseDir = view->property("projectBaseDir").toString();

    m_hugoProcess.setArguments({QStringLiteral("server"), QStringLiteral("-p"), QString::number(m_plugin->port())});
    m_hugoProcess.setWorkingDirectory(baseDir);
    m_hugoProcess.start();

    m_startHugoAction->setEnabled(false);
    m_stopHugoAction->setEnabled(true);
}

void KTextEditorPreviewView::stopHugo()
{
    m_hugoProcess.close();

    m_startHugoAction->setEnabled(true);
    m_stopHugoAction->setEnabled(false);
}

#include "moc_sitepreviewview.cpp"
